@@ -27,6 +27,8 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
+#ifdef HAVE_READLINK
 #include <X11/fonts/fntfilst.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -60,9 +62,8 @@ CatalogueAddFPE (CataloguePtr cat, FontPathElementPtr fpe)
 	    cat->fpeAlloc = 16;
 	else
 	    cat->fpeAlloc *= 2;
-	
-	new = xrealloc(cat->fpeList,
-		       cat->fpeAlloc * sizeof(FontPathElementPtr));
+
+	new = realloc(cat->fpeList, cat->fpeAlloc * sizeof(FontPathElementPtr));
 	if (new == NULL)
 	    return AllocError;
 
@@ -115,16 +116,18 @@ CatalogueUnrefFPEs (FontPathElementPtr fpe)
 	if (subfpe->refcount == 0)
 	{
 	    FontFileFreeFPE (subfpe);
-	    xfree(subfpe->name);
-	    xfree(subfpe);
+	    free(subfpe->name);
+	    free(subfpe);
 	}
     }
 
     cat->fpeCount = 0;
 }
 
+/* Rescan catalogue directory if modified timestamp has changed or
+ * the forceScan argument says to do it anyway (like on first load). */
 static int
-CatalogueRescan (FontPathElementPtr fpe)
+CatalogueRescan (FontPathElementPtr fpe, Bool forceScan)
 {
     CataloguePtr	cat = fpe->private;
     char		link[MAXFONTFILENAMELEN];
@@ -142,13 +145,13 @@ CatalogueRescan (FontPathElementPtr fpe)
     if (stat(path, &statbuf) < 0 || !S_ISDIR(statbuf.st_mode))
 	return BadFontPath;
 
-    if (statbuf.st_mtime <= cat->mtime)
+    if ((forceScan == FALSE) && (statbuf.st_mtime <= cat->mtime))
 	return Successful;
 
     dir = opendir(path);
     if (dir == NULL)
     {
-	xfree(cat);
+	free(cat);
 	return BadFontPath;
     }
 
@@ -178,7 +181,7 @@ CatalogueRescan (FontPathElementPtr fpe)
 	    len += strlen(attrib);
 	}
 
-	subfpe = xalloc(sizeof *subfpe);
+	subfpe = malloc(sizeof *subfpe);
 	if (subfpe == NULL)
 	    continue;
 
@@ -188,10 +191,10 @@ CatalogueRescan (FontPathElementPtr fpe)
 	 * (which uses font->fpe->type) goes to CatalogueCloseFont. */
 	subfpe->type = fpe->type;
 	subfpe->name_length = len;
-	subfpe->name = xalloc (len + 1);
-	if (subfpe == NULL)
+	subfpe->name = malloc (len + 1);
+	if (subfpe->name == NULL)
 	{
-	    xfree(subfpe);
+	    free(subfpe);
 	    continue;
 	}
 
@@ -205,15 +208,15 @@ CatalogueRescan (FontPathElementPtr fpe)
 
 	if (FontFileInitFPE (subfpe) != Successful)
 	{
-	    xfree(subfpe->name);
-	    xfree(subfpe);
+	    free(subfpe->name);
+	    free(subfpe);
 	    continue;
 	}
 
 	if (CatalogueAddFPE(cat, subfpe) != Successful)
 	{
 	    FontFileFreeFPE (subfpe);
-	    xfree(subfpe);
+	    free(subfpe);
 	    continue;
 	}
     }
@@ -233,7 +236,7 @@ CatalogueInitFPE (FontPathElementPtr fpe)
 {
     CataloguePtr	cat;
 
-    cat = (CataloguePtr) xalloc(sizeof *cat);
+    cat = malloc(sizeof *cat);
     if (cat == NULL)
 	return AllocError;
 
@@ -243,14 +246,14 @@ CatalogueInitFPE (FontPathElementPtr fpe)
     cat->fpeList = NULL;
     cat->mtime = 0;
 
-    return CatalogueRescan (fpe);
+    return CatalogueRescan (fpe, TRUE);
 }
 
 static int
 CatalogueResetFPE (FontPathElementPtr fpe)
 {
     /* Always just tell the caller to close and re-open */
-    return FPEResetFailed;	
+    return FPEResetFailed;
 }
 
 static int
@@ -269,37 +272,35 @@ CatalogueFreeFPE (FontPathElementPtr fpe)
 	return FontFileFreeFPE (fpe);
 
     CatalogueUnrefFPEs (fpe);
-    xfree(cat->fpeList);
-    xfree(cat);
+    free(cat->fpeList);
+    free(cat);
 
     return Successful;
 }
 
 static int
-CatalogueOpenFont (pointer client, FontPathElementPtr fpe, Mask flags, 
-		   char *name, int namelen, 
+CatalogueOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
+		   char *name, int namelen,
 		   fsBitmapFormat format, fsBitmapFormatMask fmask,
-		   XID id, FontPtr *pFont, char **aliasName, 
+		   XID id, FontPtr *pFont, char **aliasName,
 		   FontPtr non_cachable_font)
 {
     CataloguePtr cat = fpe->private;
     FontPathElementPtr subfpe;
-    FontDirectoryPtr dir;
     int i, status;
 
-    CatalogueRescan (fpe);
+    CatalogueRescan (fpe, FALSE);
 
     for (i = 0; i < cat->fpeCount; i++)
     {
 	subfpe = cat->fpeList[i];
-	dir = subfpe->private;
 	status = FontFileOpenFont(client, subfpe, flags,
 				  name, namelen, format, fmask, id,
 				  pFont, aliasName, non_cachable_font);
 	if (status == Successful || status == FontNameAlias)
 	    return status;
     }
-	    
+
     return BadFontName;
 }
 
@@ -313,38 +314,31 @@ CatalogueCloseFont (FontPathElementPtr fpe, FontPtr pFont)
 }
 
 static int
-CatalogueListFonts (pointer client, FontPathElementPtr fpe, char *pat, 
+CatalogueListFonts (pointer client, FontPathElementPtr fpe, char *pat,
 		    int len, int max, FontNamesPtr names)
 {
     CataloguePtr cat = fpe->private;
     FontPathElementPtr subfpe;
-    FontDirectoryPtr dir;
     int i;
 
-    CatalogueRescan (fpe);
+    CatalogueRescan (fpe, FALSE);
 
     for (i = 0; i < cat->fpeCount; i++)
     {
 	subfpe = cat->fpeList[i];
-	dir = subfpe->private;
 	FontFileListFonts(client, subfpe, pat, len, max, names);
     }
 
     return Successful;
 }
 
-int
-FontFileStartListFonts(pointer client, FontPathElementPtr fpe, 
-		       char *pat, int len, int max, 
-		       pointer *privatep, int mark_aliases);
-
 typedef struct _LFWIData {
     pointer		*privates;
     int			current;
 } LFWIDataRec, *LFWIDataPtr;
 
 static int
-CatalogueStartListFonts(pointer client, FontPathElementPtr fpe, 
+CatalogueStartListFonts(pointer client, FontPathElementPtr fpe,
 			char *pat, int len, int max, pointer *privatep,
 			int mark_aliases)
 {
@@ -352,10 +346,9 @@ CatalogueStartListFonts(pointer client, FontPathElementPtr fpe,
     LFWIDataPtr		data;
     int			ret, i, j;
 
-    CatalogueRescan (fpe);
+    CatalogueRescan (fpe, FALSE);
 
-    data = (LFWIDataPtr) xalloc (sizeof *data +
-				 sizeof data->privates[0] * cat->fpeCount);
+    data = malloc (sizeof *data + sizeof data->privates[0] * cat->fpeCount);
     if (!data)
 	return AllocError;
     data->privates = (pointer *) (data + 1);
@@ -375,22 +368,22 @@ CatalogueStartListFonts(pointer client, FontPathElementPtr fpe,
  bail:
     for (j = 0; j < i; j++)
 	/* FIXME: we have no way to free the per-fpe privates. */;
-    xfree (data);
+    free (data);
 
     return AllocError;
 }
 
 static int
-CatalogueStartListFontsWithInfo(pointer client, FontPathElementPtr fpe, 
-				char *pat, int len, int max, 
+CatalogueStartListFontsWithInfo(pointer client, FontPathElementPtr fpe,
+				char *pat, int len, int max,
 				pointer *privatep)
 {
     return CatalogueStartListFonts(client, fpe, pat, len, max, privatep, 0);
 }
 
 static int
-CatalogueListNextFontWithInfo(pointer client, FontPathElementPtr fpe, 
-			      char **namep, int *namelenp, 
+CatalogueListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
+			      char **namep, int *namelenp,
 			      FontInfoPtr *pFontInfo,
 			      int *numFonts, pointer private)
 {
@@ -400,7 +393,7 @@ CatalogueListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
 
     if (data->current == cat->fpeCount)
     {
-	xfree(data);
+	free(data);
 	return BadFontName;
     }
 
@@ -419,15 +412,15 @@ CatalogueListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
 }
 
 static int
-CatalogueStartListFontsAndAliases(pointer client, FontPathElementPtr fpe, 
-				  char *pat, int len, int max, 
+CatalogueStartListFontsAndAliases(pointer client, FontPathElementPtr fpe,
+				  char *pat, int len, int max,
 				  pointer *privatep)
 {
     return CatalogueStartListFonts(client, fpe, pat, len, max, privatep, 1);
 }
 
 static int
-CatalogueListNextFontOrAlias(pointer client, FontPathElementPtr fpe, 
+CatalogueListNextFontOrAlias(pointer client, FontPathElementPtr fpe,
 			     char **namep, int *namelenp, char **resolvedp,
 			     int *resolvedlenp, pointer private)
 {
@@ -437,7 +430,7 @@ CatalogueListNextFontOrAlias(pointer client, FontPathElementPtr fpe,
 
     if (data->current == cat->fpeCount)
     {
-	xfree(data);
+	free(data);
 	return BadFontName;
     }
 
@@ -474,3 +467,5 @@ CatalogueRegisterLocalFpeFunctions (void)
 			 CatalogueListNextFontOrAlias,
 			 FontFileEmptyBitmapSource);
 }
+
+#endif /* HAVE_READLINK */
